// WIN32  EBhERg[pNXQ
#ifndef _TTWINDOW_H_
#define _TTWINDOW_H_

#include <windows.h>
#include <commctrl.h>

class TtWindow;
class TtStatic;
class TtButton;
class TtCheckBox;
class TtGroup;
class TtEdit;
class TtList;
class TtScroll;
class TtScrollV;
class TtScrollH;
class TtTab;
class TtApp;

// -- TtWindow -----------------------------------------------------------
class TtWindow {
  
  // -- static -----------------------------------------------------------
public:
  static const int M_DEFAULT = 0x8000;
  
  // -- message dispatch system ------------------------------------------
private:
  static int wnd_count;
  static int ary_max;
  static HWND *h_wnd_ary;
  static TtWindow **instance_ary;

public:
  static LRESULT CALLBACK _ttl_wnd_proc( HWND h_wnd, UINT msg, WPARAM w_param, LPARAM l_param );
  static TtWindow *find_window( HWND h_wnd );
  static void init_common_controls();

  // -- override at sub class --------------------------------------------
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
  virtual bool created();

public:
  virtual LRESULT wnd_proc( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );

  // -- instance ---------------------------------------------------------
protected:
  TtWindow *parent;
  HWND h_wnd;
  int id;
  HINSTANCE h_instance;
  WNDPROC wnd_proc_super;

  int override_wnd_proc();

public:
  TtWindow();
  bool create( TtWindow *parent, int id, HINSTANCE h_instance, LPVOID lp_param );
  int close();
  
  HWND get_h_wnd();
  int get_id();
  HINSTANCE get_h_instance();
  
  TtWindow *get_parent();
  RECT get_rect();
  int get_pos_x();
  int get_pos_y();
  int get_width();
  int get_height();
  
  int show();
  int hide();
  int minimize();
  int maximize();
  int restore();
  int set_show_state( int state );
  
  int set_position( int x, int y );
  int set_size( int width, int height );
  int set_pos_size( int x, int y, int width, int height );
  int set_rect( RECT rect );

  bool set_bottom();
  bool set_top();
  bool set_under( HWND wnd );
  
  int set_font( HFONT h_font );

  int set_forground();

  int get_text( char *buf, int buf_size );
  int set_text( const char *str );

  int messageloop();
};

// -- TtStatic -----------------------------------------------------------
class TtStatic : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtButton -----------------------------------------------------------
class TtButton : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtCheckBox ----------------------------------------------------------
class TtCheckBox : public TtButton {
protected:
  virtual DWORD  get_style();

public:
  bool get_check();
  int set_check( bool checking );
};

// -- TtGroup ------------------------------------------------------------
class TtGroup : public TtButton {
protected:
  virtual DWORD  get_style();
};

// -- TtEdit -------------------------------------------------------------
class TtEdit : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
};

// -- TtList -------------------------------------------------------------
class TtList : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_count();
  int get_text_at( int index, char *buf );
  int get_current();
  int find( int start, const char *str );

  int push( const char *str );
  int insert_at( int index, const char *str );

  int delete_at( int index );
  void clear();

  int get_selected_count();
  bool is_select( int index );
  int set_select( int index );
  int unset_select( int index );
  
  int lock_draw();
  int unlock_draw();
};

// -- TtScroll -----------------------------------------------------------
class TtScroll : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

  SCROLLINFO info;

public:

  virtual int get_track_position();
  virtual void set_track_position( int position );

  virtual int get_min();
  virtual void set_min( int min );
  virtual int get_max();
  virtual void set_max( int max );
  virtual void set_range( int min, int max );

  virtual int get_page_size();
  virtual void set_page_size( int page_size );
  
protected:
  bool get_info();
  void set_info();

  LRESULT scrolling( int code, int position );
};

// -- TtScrollV ----------------------------------------------------------
class TtScrollV : public TtScroll {
protected:
  virtual DWORD  get_style();
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );
};

// -- TtScrollH ----------------------------------------------------------
class TtScrollH : public TtScroll {
protected:
  virtual DWORD  get_style();
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
};

// -- TtTrack ------------------------------------------------------------
class TtTrack : public TtScroll {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  virtual int get_track_position();
  virtual void set_track_position( int position );

  virtual int get_min();
  virtual void set_min( int min );
  virtual int get_max();
  virtual void set_max( int max );
  virtual void set_range( int min, int max );

  virtual int get_page_size();
  virtual void set_page_size( int page_size );

  virtual int get_line_size();
  virtual void set_line_size( int line_size );

  void set_tic_frequency( int frequency );
  void clear_tic();
  
protected:
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );

};

// -- TtTrackV -----------------------------------------------------------
class TtTrackV : public TtTrack {
protected:
  virtual DWORD  get_style();
};

// -- TtTrackH -----------------------------------------------------------
class TtTrackH : public TtTrack {
protected:
  virtual DWORD  get_style();
};

// -- TtTab --------------------------------------------------------------
class TtTab : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_count();
  
  int insert_at( int index, const char *str );

  int delete_at( int index );
  int clear();

  int set_current( int index );
  int get_current();

  RECT get_view_rect();
};

// -- TtProgress ---------------------------------------------------------
class TtProgress : public TtWindow {
protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();

public:
  int get_max();
  int get_amount();
  
  int set_max( int max );
  int set_step( int step );
  
  int set_amount( int position );
  
  int step_up( int amount );
  int step_it();
};

// -- TtApp --------------------------------------------------------------
class TtApp : public TtWindow {
private:
  static char *class_name;

public:
  static bool init( HINSTANCE h_instance );

protected:
  virtual DWORD  get_ex_style();
  virtual DWORD  get_style();
  virtual PCTSTR get_class_name();
  virtual bool created();

  virtual LRESULT wm_command_recved( int id, int code, HWND child );
  virtual LRESULT wm_notify_recved( int id, NMHDR *nmhdr );
  virtual LRESULT wm_size_recved( int flag, int width, int height );
  virtual LRESULT wm_destroy_recved();

public:
  virtual LRESULT wnd_proc( UINT msg, WPARAM w_param, LPARAM l_param );
  virtual LRESULT wm_hscroll_recved( int code, int position, HWND h_wnd );
  virtual LRESULT wm_vscroll_recved( int code, int position, HWND h_wnd );

};

#endif // _TTWINDOW_H_
